@@ -8,7 +8,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -41,7 +41,7 @@
 #  include "config.h"
 #endif
 
-#include <slurm/slurm.h>
+#include "slurm/slurm.h"
 #include "src/common/slurm_protocol_api.h"
 
 static int _suspend_op (uint16_t op, uint32_t job_id);
