@@ -9,7 +9,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -57,7 +57,7 @@
 #include <sys/types.h>
 #include <stdarg.h>
 
-#include <slurm/slurm_errno.h>
+#include "slurm/slurm_errno.h"
 
 #include "src/common/pack.h"
 #include "src/common/slurm_protocol_common.h"
@@ -90,13 +90,13 @@ enum {
  * sets the slurm_protocol_config object
  * IN protocol_conf		-  slurm_protocol_config object
  */
-inline int slurm_set_api_config(slurm_protocol_config_t * protocol_conf);
+extern int slurm_set_api_config(slurm_protocol_config_t * protocol_conf);
 
 /* slurm_get_api_config
  * returns a pointer to the current slurm_protocol_config object
  * RET slurm_protocol_config_t	- current slurm_protocol_config object
  */
-inline slurm_protocol_config_t *slurm_get_api_config(void);
+extern slurm_protocol_config_t *slurm_get_api_config(void);
 
 /* slurm_get_batch_start_timeout
  * RET BatchStartTimeout value from slurm.conf
@@ -128,6 +128,10 @@ uint16_t slurm_get_complete_wait(void);
  */
 uint32_t slurm_get_debug_flags(void);
 
+/* slurm_set_debug_flags
+ */
+void slurm_set_debug_flags(uint32_t debug_flags);
+
 /* slurm_get_def_mem_per_cpu
  * RET DefMemPerCPU/Node value from slurm.conf
  */
@@ -151,7 +155,7 @@ uint32_t slurm_get_epilog_msg_time(void);
 /* slurm_get_env_timeout
  * return default timeout for srun/sbatch --get-user-env option
  */
-inline int slurm_get_env_timeout(void);
+extern int slurm_get_env_timeout(void);
 
 /* slurm_get_mpi_default
  * get default mpi value from slurmctld_conf object
@@ -168,7 +172,7 @@ char *slurm_get_mpi_params(void);
 /* slurm_get_msg_timeout
  * get default message timeout value from slurmctld_conf object
  */
-inline uint16_t slurm_get_msg_timeout(void);
+extern uint16_t slurm_get_msg_timeout(void);
 
 /* slurm_api_set_conf_file
  *      set slurm configuration file to a non-default value
@@ -181,11 +185,11 @@ extern void slurm_api_set_conf_file(char *pathname);
  *	the compiled in default slurm_protocol_config object is initialized
  * RET int 		- return code
  */
-inline int slurm_api_set_default_config();
+extern int slurm_api_set_default_config();
 
 /* slurm_api_clear_config
  * execute this only at program termination to free all memory */
-inline void slurm_api_clear_config(void);
+extern void slurm_api_clear_config(void);
 
 /* slurm_get_hash_val
  * get hash val of the slurm.conf from slurmctld_conf object from
@@ -531,7 +535,7 @@ extern uint16_t slurm_get_sched_port(void);
  * returns slurmd port from slurmctld_conf object
  * RET uint16_t	- slurmd port
  */
-inline uint16_t slurm_get_slurmd_port(void);
+extern uint16_t slurm_get_slurmd_port(void);
 
 /* slurm_get_slurm_user_id
  * returns slurm uid from slurmctld_conf object
@@ -557,6 +561,12 @@ char *slurm_get_sched_type(void);
  */
 char *slurm_get_select_type(void);
 
+/* slurm_get_select_type_param
+ * get select_type_param from slurmctld_conf object
+ * RET uint16_t   - select_type_param
+ */
+uint16_t slurm_get_select_type_param(void);
+
 /** Return true if (remote) system runs Cray XT/XE */
 bool is_cray_select_type(void);
 
@@ -609,7 +619,7 @@ uint16_t slurm_get_task_plugin_param(void);
  * IN port		- port to bind the msg server to
  * RET slurm_fd		- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_init_msg_engine_port(uint16_t port);
+extern slurm_fd_t slurm_init_msg_engine_port(uint16_t port);
 
 /* In the socket implementation it creates a socket, binds to it, and
  *	listens for connections.
@@ -618,7 +628,7 @@ inline slurm_fd_t slurm_init_msg_engine_port(uint16_t port);
  * IN port		- port to bind the msg server to
  * RET slurm_fd		- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_init_msg_engine_addrname_port(char *addr_name,
+extern slurm_fd_t slurm_init_msg_engine_addrname_port(char *addr_name,
 						    uint16_t port);
 
 /* In the socket implementation it creates a socket, binds to it, and
@@ -626,14 +636,14 @@ inline slurm_fd_t slurm_init_msg_engine_addrname_port(char *addr_name,
  * IN slurm_address 	- slurm_addr_t to bind the msg server to
  * RET slurm_fd		- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_init_msg_engine(slurm_addr_t * slurm_address);
+extern slurm_fd_t slurm_init_msg_engine(slurm_addr_t * slurm_address);
 
 /* In the bsd implmentation maps directly to a accept call
  * IN open_fd		- file descriptor to accept connection on
  * OUT slurm_address 	- slurm_addr_t of the accepted connection
  * RET slurm_fd		- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_accept_msg_conn(slurm_fd_t open_fd,
+extern slurm_fd_t slurm_accept_msg_conn(slurm_fd_t open_fd,
 				      slurm_addr_t * slurm_address);
 
 /* In the bsd implmentation maps directly to a close call, to close
@@ -641,13 +651,13 @@ inline slurm_fd_t slurm_accept_msg_conn(slurm_fd_t open_fd,
  * IN open_fd		- an open file descriptor to close
  * RET int		- the return code
  */
-inline int slurm_close_accepted_conn(slurm_fd_t open_fd);
+extern int slurm_close_accepted_conn(slurm_fd_t open_fd);
 
 /* just calls close on an established msg connection
  * IN open_fd	- an open file descriptor to close
  * RET int	- the return code
  */
-inline int slurm_shutdown_msg_engine(slurm_fd_t open_fd);
+extern int slurm_shutdown_msg_engine(slurm_fd_t open_fd);
 
 /**********************************************************************\
  * receive message functions
@@ -727,8 +737,8 @@ int slurm_send_node_msg(slurm_fd_t open_fd, slurm_msg_t *msg);
  * IN/OUT addr     - address of controller contacted
  * RET slurm_fd	- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_open_controller_conn(slurm_addr_t *addr);
-inline slurm_fd_t slurm_open_controller_conn_spec(enum controller_id dest);
+extern slurm_fd_t slurm_open_controller_conn(slurm_addr_t *addr);
+extern slurm_fd_t slurm_open_controller_conn_spec(enum controller_id dest);
 /* gets the slurm_addr_t of the specified controller
  *	primary or secondary slurmctld message engine
  * IN dest      - controller to contact, primary or secondary
@@ -745,13 +755,13 @@ void slurm_get_controller_addr_spec(enum controller_id dest,
  * IN slurm_address 	- slurm_addr_t of the connection destination
  * RET slurm_fd		- file descriptor of the connection created
  */
-inline slurm_fd_t slurm_open_msg_conn(slurm_addr_t * slurm_address);
+extern slurm_fd_t slurm_open_msg_conn(slurm_addr_t * slurm_address);
 
 /* just calls close on an established msg connection to close
  * IN open_fd	- an open file descriptor to close
  * RET int	- the return code
  */
-inline int slurm_shutdown_msg_conn(slurm_fd_t open_fd);
+extern int slurm_shutdown_msg_conn(slurm_fd_t open_fd);
 
 
 /**********************************************************************\
@@ -763,7 +773,7 @@ inline int slurm_shutdown_msg_conn(slurm_fd_t open_fd);
  * IN slurm_address 	- slurm_addr_t to bind the server stream to
  * RET slurm_fd		- file descriptor of the stream created
  */
-inline slurm_fd_t slurm_listen_stream(slurm_addr_t * slurm_address);
+extern slurm_fd_t slurm_listen_stream(slurm_addr_t * slurm_address);
 
 /* slurm_accept_stream
  * accepts a incoming stream connection on a stream server slurm_fd
@@ -771,7 +781,7 @@ inline slurm_fd_t slurm_listen_stream(slurm_addr_t * slurm_address);
  * OUT slurm_address 	- slurm_addr_t of the accepted connection
  * RET slurm_fd		- file descriptor of the accepted connection
  */
-inline slurm_fd_t slurm_accept_stream(slurm_fd_t open_fd,
+extern slurm_fd_t slurm_accept_stream(slurm_fd_t open_fd,
 				    slurm_addr_t * slurm_address);
 
 /* slurm_open_stream
@@ -779,14 +789,14 @@ inline slurm_fd_t slurm_accept_stream(slurm_fd_t open_fd,
  * IN slurm_address 	- slurm_addr_t of the connection destination
  * RET slurm_fd_t         - file descriptor of the connection created
  */
-inline slurm_fd_t slurm_open_stream(slurm_addr_t * slurm_address);
+extern slurm_fd_t slurm_open_stream(slurm_addr_t * slurm_address);
 
 /* slurm_close_stream
  * closes either a server or client stream file_descriptor
  * IN open_fd	- an open file descriptor to close
  * RET int	- the return code
  */
-inline int slurm_close_stream(slurm_fd_t open_fd);
+extern int slurm_close_stream(slurm_fd_t open_fd);
 
 /* slurm_write_stream
  * writes a buffer out a stream file descriptor
@@ -796,9 +806,9 @@ inline int slurm_close_stream(slurm_fd_t open_fd);
  * IN timeout		- how long to wait in milliseconds
  * RET size_t		- bytes sent , or -1 on errror
  */
-inline size_t slurm_write_stream(slurm_fd_t open_fd, char *buffer,
+extern size_t slurm_write_stream(slurm_fd_t open_fd, char *buffer,
 				 size_t size);
-inline size_t slurm_write_stream_timeout(slurm_fd_t open_fd,
+extern size_t slurm_write_stream_timeout(slurm_fd_t open_fd,
 					 char *buffer, size_t size,
 					 int timeout);
 
@@ -810,9 +820,9 @@ inline size_t slurm_write_stream_timeout(slurm_fd_t open_fd,
  * IN timeout		- how long to wait in milliseconds
  * RET size_t		- bytes read , or -1 on errror
  */
-inline size_t slurm_read_stream(slurm_fd_t open_fd, char *buffer,
+extern size_t slurm_read_stream(slurm_fd_t open_fd, char *buffer,
 				size_t size);
-inline size_t slurm_read_stream_timeout(slurm_fd_t open_fd,
+extern size_t slurm_read_stream_timeout(slurm_fd_t open_fd,
 					char *buffer, size_t size,
 					int timeout);
 
@@ -821,15 +831,15 @@ inline size_t slurm_read_stream_timeout(slurm_fd_t open_fd,
  * IN open_fd 		- file descriptor to retreive slurm_addr_t for
  * OUT address		- address that open_fd to bound to
  */
-inline int slurm_get_stream_addr(slurm_fd_t open_fd, slurm_addr_t * address);
+extern int slurm_get_stream_addr(slurm_fd_t open_fd, slurm_addr_t * address);
 
 /* make an open slurm connection blocking or non-blocking
  *	(i.e. wait or do not wait for i/o completion )
  * IN open_fd	- an open file descriptor to change the effect
  * RET int	- the return code
  */
-inline int slurm_set_stream_non_blocking(slurm_fd_t open_fd);
-inline int slurm_set_stream_blocking(slurm_fd_t open_fd);
+extern int slurm_set_stream_non_blocking(slurm_fd_t open_fd);
+extern int slurm_set_stream_blocking(slurm_fd_t open_fd);
 
 /**********************************************************************\
  * address conversion and management functions
@@ -841,7 +851,7 @@ inline int slurm_set_stream_blocking(slurm_fd_t open_fd);
  * IN port		- port in host order
  * IN ip_address	- ipv4 address in uint32 host order form
  */
-inline void slurm_set_addr_uint(slurm_addr_t * slurm_address,
+extern void slurm_set_addr_uint(slurm_addr_t * slurm_address,
 				uint16_t port, uint32_t ip_address);
 
 /* reset_slurm_addr
@@ -857,7 +867,7 @@ void reset_slurm_addr(slurm_addr_t * slurm_address, slurm_addr_t new_address);
  * IN port		- port in host order
  * IN host		- hostname or dns name
  */
-inline void slurm_set_addr(slurm_addr_t * slurm_address,
+extern void slurm_set_addr(slurm_addr_t * slurm_address,
 			   uint16_t port, char *host);
 
 /* slurm_set_addr_any
@@ -865,7 +875,7 @@ inline void slurm_set_addr(slurm_addr_t * slurm_address,
  * OUT slurm_address	- slurm_addr_t to be filled in
  * IN port		- port in host order
  */
-inline void slurm_set_addr_any(slurm_addr_t * slurm_address, uint16_t port);
+extern void slurm_set_addr_any(slurm_addr_t * slurm_address, uint16_t port);
 
 /* slurm_set_addr_char
  * initializes the slurm_address with the supplied port and host
@@ -873,7 +883,7 @@ inline void slurm_set_addr_any(slurm_addr_t * slurm_address, uint16_t port);
  * IN port		- port in host order
  * IN host		- hostname or dns name
  */
-inline void slurm_set_addr_char(slurm_addr_t * slurm_address,
+extern void slurm_set_addr_char(slurm_addr_t * slurm_address,
 				uint16_t port, char *host);
 
 /* slurm_get_addr
@@ -883,7 +893,7 @@ inline void slurm_set_addr_char(slurm_addr_t * slurm_address,
  * OUT host		- hostname
  * IN buf_len		- length of hostname buffer
  */
-inline void slurm_get_addr(slurm_addr_t * slurm_address,
+extern void slurm_get_addr(slurm_addr_t * slurm_address,
 			   uint16_t * port, char *host, uint32_t buf_len);
 
 /* slurm_get_ip_str
@@ -893,7 +903,7 @@ inline void slurm_get_addr(slurm_addr_t * slurm_address,
  * OUT ip		- ip address in dotted-quad string form
  * IN buf_len		- length of ip buffer
  */
-inline void slurm_get_ip_str(slurm_addr_t * slurm_address, uint16_t * port,
+extern void slurm_get_ip_str(slurm_addr_t * slurm_address, uint16_t * port,
 			     char *ip, unsigned int buf_len);
 
 /* slurm_get_peer_addr
@@ -901,7 +911,7 @@ inline void slurm_get_ip_str(slurm_addr_t * slurm_address, uint16_t * port,
  * IN fd		- an open connection
  * OUT slurm_address	- place to park the peer's slurm_addr
  */
-inline int slurm_get_peer_addr(slurm_fd_t fd, slurm_addr_t * slurm_address);
+extern int slurm_get_peer_addr(slurm_fd_t fd, slurm_addr_t * slurm_address);
 
 /* slurm_print_slurm_addr
  * prints a slurm_addr_t into a buf
@@ -909,21 +919,19 @@ inline int slurm_get_peer_addr(slurm_fd_t fd, slurm_addr_t * slurm_address);
  * IN buf		- space for string representation of slurm_addr
  * IN n			- max number of bytes to write (including NUL)
  */
-inline void slurm_print_slurm_addr(slurm_addr_t * address,
+extern void slurm_print_slurm_addr(slurm_addr_t * address,
 				   char *buf, size_t n);
 
 /**********************************************************************\
  * slurm_addr_t pack routines
 \**********************************************************************/
 
-Buf slurm_pack_msg_no_header(slurm_msg_t * msg);
-
 /* slurm_pack_slurm_addr
  * packs a slurm_addr_t into a buffer to serialization transport
  * IN slurm_address	- slurm_addr_t to pack
  * IN/OUT buffer	- buffer to pack the slurm_addr_t into
  */
-inline void slurm_pack_slurm_addr(slurm_addr_t * slurm_address, Buf buffer);
+extern void slurm_pack_slurm_addr(slurm_addr_t * slurm_address, Buf buffer);
 
 /* slurm_pack_slurm_addr
  * unpacks a buffer into a slurm_addr_t after serialization transport
@@ -931,7 +939,7 @@ inline void slurm_pack_slurm_addr(slurm_addr_t * slurm_address, Buf buffer);
  * IN/OUT buffer	- buffer to upack the slurm_addr_t from
  * returns 		- SLURM error code
  */
-inline int slurm_unpack_slurm_addr_no_alloc(slurm_addr_t * slurm_address,
+extern int slurm_unpack_slurm_addr_no_alloc(slurm_addr_t * slurm_address,
 					    Buf buffer);
 
 /* slurm_pack_slurm_addr_array
@@ -941,7 +949,7 @@ inline int slurm_unpack_slurm_addr_no_alloc(slurm_addr_t * slurm_address,
  * IN/OUT buffer	- buffer to pack the slurm_addr_t from
  * returns		- SLURM error code
  */
-inline void slurm_pack_slurm_addr_array(slurm_addr_t * slurm_address,
+extern void slurm_pack_slurm_addr_array(slurm_addr_t * slurm_address,
 					uint32_t size_val, Buf buffer);
 /* slurm_unpack_slurm_addr_array
  * unpacks an array of slurm_addrs from a buffer
@@ -950,7 +958,7 @@ inline void slurm_pack_slurm_addr_array(slurm_addr_t * slurm_address,
  * IN/OUT buffer	- buffer to upack the slurm_addr_t from
  * returns		- SLURM error code
  */
-inline int slurm_unpack_slurm_addr_array(slurm_addr_t ** slurm_address,
+extern int slurm_unpack_slurm_addr_array(slurm_addr_t ** slurm_address,
 					 uint32_t * size_val, Buf buffer);
 
 /**********************************************************************\
@@ -995,7 +1003,7 @@ int slurm_send_recv_node_msg(slurm_msg_t * request_msg,
  * IN nodelist	    - list of nodes to send to.
  * IN msg           - a slurm_msg struct to be sent by the function
  * IN timeout	    - how long to wait in milliseconds
- * IN quiet       - if set, reduce logging details
+ * IN quiet         - if set, reduce logging details
  * RET List	    - List containing the responses of the childern
  *                    (if any) we forwarded the message to. List
  *                    containing type (ret_types_t).
