@@ -9,7 +9,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -39,12 +39,10 @@
 #  include <string.h>
 #endif                /* HAVE_CONFIG_H */
 
-
-#include <slurm/slurm.h>
-
 #include <stdlib.h>
 
-#include <slurm/slurm_errno.h>
+#include "slurm/slurm.h"
+#include "slurm/slurm_errno.h"
 
 #include "src/common/slurm_step_layout.h"
 #include "src/common/log.h"
@@ -286,21 +284,38 @@ extern void pack_slurm_step_layout(slurm_step_layout_t *step_layout,
 				   Buf buffer, uint16_t protocol_version)
 {
 	uint16_t i = 0;
-	if(protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
-		if(step_layout)
+
+	if (protocol_version >= SLURM_2_3_PROTOCOL_VERSION) {
+		if (step_layout)
 			i=1;
 
 		pack16(i, buffer);
-		if(!i)
+		if (!i)
+			return;
+		packstr(step_layout->front_end, buffer);
+		packstr(step_layout->node_list, buffer);
+		pack32(step_layout->node_cnt, buffer);
+		pack32(step_layout->task_cnt, buffer);
+		pack16(step_layout->task_dist, buffer);
+
+		for (i=0; i<step_layout->node_cnt; i++) {
+			pack32_array(step_layout->tids[i],
+				     step_layout->tasks[i],
+				     buffer);
+		}
+	} else if (protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
+		if (step_layout)
+			i=1;
+
+		pack16(i, buffer);
+		if (!i)
 			return;
 		packstr(step_layout->node_list, buffer);
 		pack32(step_layout->node_cnt, buffer);
 		pack32(step_layout->task_cnt, buffer);
 		pack16(step_layout->task_dist, buffer);
-/* 	slurm_pack_slurm_addr_array(step_layout->node_addr,  */
-/* 				    step_layout->node_cnt, buffer); */
 
-		for(i=0; i<step_layout->node_cnt; i++) {
+		for (i=0; i<step_layout->node_cnt; i++) {
 			pack32_array(step_layout->tids[i],
 				     step_layout->tasks[i],
 				     buffer);
@@ -316,35 +331,51 @@ extern int unpack_slurm_step_layout(slurm_step_layout_t **layout, Buf buffer,
 	slurm_step_layout_t *step_layout = NULL;
 	int i;
 
-	if(protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_2_3_PROTOCOL_VERSION) {
 		safe_unpack16(&uint16_tmp, buffer);
-		if(!uint16_tmp)
+		if (!uint16_tmp)
 			return SLURM_SUCCESS;
 
 		step_layout = xmalloc(sizeof(slurm_step_layout_t));
 		*layout = step_layout;
 
-		step_layout->node_list = NULL;
-		step_layout->node_cnt = 0;
-		step_layout->tids = NULL;
-		step_layout->tasks = NULL;
+		safe_unpackstr_xmalloc(&step_layout->front_end,
+				       &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&step_layout->node_list,
 				       &uint32_tmp, buffer);
 		safe_unpack32(&step_layout->node_cnt, buffer);
 		safe_unpack32(&step_layout->task_cnt, buffer);
 		safe_unpack16(&step_layout->task_dist, buffer);
 
-/* 	if (slurm_unpack_slurm_addr_array(&(step_layout->node_addr),  */
-/* 					  &uint32_tmp, buffer)) */
-/* 		goto unpack_error; */
-/* 	if (uint32_tmp != step_layout->node_cnt) */
-/* 		goto unpack_error; */
+		step_layout->tasks =
+			xmalloc(sizeof(uint32_t) * step_layout->node_cnt);
+		step_layout->tids = xmalloc(sizeof(uint32_t *)
+					    * step_layout->node_cnt);
+		for (i = 0; i < step_layout->node_cnt; i++) {
+			safe_unpack32_array(&(step_layout->tids[i]),
+					    &num_tids,
+					    buffer);
+			step_layout->tasks[i] = num_tids;
+		}
+	} else if (protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
+		safe_unpack16(&uint16_tmp, buffer);
+		if (!uint16_tmp)
+			return SLURM_SUCCESS;
+
+		step_layout = xmalloc(sizeof(slurm_step_layout_t));
+		*layout = step_layout;
+
+		safe_unpackstr_xmalloc(&step_layout->node_list,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&step_layout->node_cnt, buffer);
+		safe_unpack32(&step_layout->task_cnt, buffer);
+		safe_unpack16(&step_layout->task_dist, buffer);
 
 		step_layout->tasks =
 			xmalloc(sizeof(uint32_t) * step_layout->node_cnt);
 		step_layout->tids = xmalloc(sizeof(uint32_t *)
 					    * step_layout->node_cnt);
-		for(i = 0; i < step_layout->node_cnt; i++) {
+		for (i = 0; i < step_layout->node_cnt; i++) {
 			safe_unpack32_array(&(step_layout->tids[i]),
 					    &num_tids,
 					    buffer);
@@ -363,9 +394,9 @@ unpack_error:
 extern int slurm_step_layout_destroy(slurm_step_layout_t *step_layout)
 {
 	int i=0;
-	if(step_layout) {
+	if (step_layout) {
+		xfree(step_layout->front_end);
 		xfree(step_layout->node_list);
-/* 		xfree(step_layout->node_addr); */
 		xfree(step_layout->tasks);
 		for (i = 0; i < step_layout->node_cnt; i++) {
 			xfree(step_layout->tids[i]);
@@ -419,7 +450,7 @@ static int _init_task_layout(slurm_step_layout_t *step_layout,
 	if (step_layout->tasks)	/* layout already completed */
 		return SLURM_SUCCESS;
 
-	if((int)cpus_per_task < 1 || cpus_per_task == (uint16_t)NO_VAL)
+	if ((int)cpus_per_task < 1 || cpus_per_task == (uint16_t)NO_VAL)
 		cpus_per_task = 1;
 
 	step_layout->plane_size = plane_size;
@@ -428,26 +459,26 @@ static int _init_task_layout(slurm_step_layout_t *step_layout,
 				     * step_layout->node_cnt);
 	step_layout->tids  = xmalloc(sizeof(uint32_t *)
 				     * step_layout->node_cnt);
-	if(!(cluster_flags & CLUSTER_FLAG_BG)) {
+	if (!(cluster_flags & CLUSTER_FLAG_BG)) {
 		hostlist_t hl = hostlist_create(step_layout->node_list);
 		/* make sure the number of nodes we think we have
 		 * is the correct number */
 		i = hostlist_count(hl);
-		if(step_layout->node_cnt > i)
+		if (step_layout->node_cnt > i)
 			step_layout->node_cnt = i;
 		hostlist_destroy(hl);
 	}
-	debug("laying out the %u tasks on %u hosts %s",
+	debug("laying out the %u tasks on %u hosts %s dist %u",
 	      step_layout->task_cnt, step_layout->node_cnt,
-	      step_layout->node_list);
-	if(step_layout->node_cnt < 1) {
+	      step_layout->node_list, task_dist);
+	if (step_layout->node_cnt < 1) {
 		error("no hostlist given can't layout tasks");
 		return SLURM_ERROR;
 	}
 
 	for (i=0; i<step_layout->node_cnt; i++) {
 /* 		name = hostlist_shift(hl); */
-/* 		if(!name) { */
+/* 		if (!name) { */
 /* 			error("hostlist incomplete for this job request"); */
 /* 			hostlist_destroy(hl); */
 /* 			return SLURM_ERROR; */
