@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  mysql_common.h - common functions for the the mysql storage plugin.
+ *  mysql_common.h - common functions for the mysql storage plugin.
  *****************************************************************************
  *
  *  Copyright (C) 2004-2007 The Regents of the University of California.
@@ -7,7 +7,7 @@
  *  Written by Danny Auble <da@llnl.gov>
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -57,7 +57,7 @@
 #endif
 
 #include <stdio.h>
-#include <slurm/slurm_errno.h>
+#include "slurm/slurm_errno.h"
 #include "src/common/list.h"
 #include "src/common/xstring.h"
 #include <mysql.h>
