@@ -8,7 +8,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -39,6 +39,7 @@
 
 #include "./msg.h"
 #include "src/common/hostlist.h"
+#include "src/common/node_select.h"
 #include "src/slurmctld/locks.h"
 #include "src/slurmctld/slurmctld.h"
 
@@ -85,6 +86,28 @@ extern int	get_nodes(char *cmd_ptr, int *err_code, char **err_msg)
 		NO_LOCK, NO_LOCK, READ_LOCK, READ_LOCK };
 	int node_rec_cnt = 0, buf_size = 0;
 
+#ifdef HAVE_CRAY
+	/* Locks: write node */
+	slurmctld_lock_t node_write_lock = {
+		NO_LOCK, NO_LOCK, WRITE_LOCK, NO_LOCK };
+
+	/*
+	 * Run a Basil Inventory immediately before scheduling, to avoid
+	 * race conditions caused by ALPS node state change (caused e.g.
+	 * by the node health checker).
+	 * This relies on the above write lock for the node state.
+	 */
+	lock_slurmctld(node_write_lock);
+	if (select_g_reconfigure()) {
+		unlock_slurmctld(node_write_lock);
+		*err_code = -720;
+		*err_msg = "Unable to run ALPS inventory";
+		error("wiki: Unable to run ALPS inventory");
+		return -1;
+	}
+	unlock_slurmctld(node_write_lock);
+#endif
+
 	arg_ptr = strstr(cmd_ptr, "ARG=");
 	if (arg_ptr == NULL) {
 		*err_code = -300;
