@@ -6,7 +6,7 @@
  *  Copyright (C) 2005 Hewlett-Packard Development Company, L.P.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -53,10 +53,10 @@ print_field_t fields[] = {
 	{10, "AveVMSize", print_fields_str, PRINT_AVEVSIZE},
 	{16, "BlockID", print_fields_str, PRINT_BLOCKID},
 	{10, "Cluster", print_fields_str, PRINT_CLUSTER},
+	{14, "Comment", print_fields_str, PRINT_COMMENT},
 	{10, "CPUTime", print_fields_time_from_secs, PRINT_CPU_TIME},
 	{10, "CPUTimeRAW", print_fields_int, PRINT_CPU_TIME_RAW},
 	{15, "DerivedExitCode", print_fields_str, PRINT_DERIVED_EC},
-	{14, "DerivedExitStr", print_fields_str, PRINT_DERIVED_ES},
 	{10, "Elapsed", print_fields_time_from_secs, PRINT_ELAPSED},
 	{19, "Eligible", print_fields_date, PRINT_ELIGIBLE},
 	{19, "End", print_fields_date, PRINT_END},
