@@ -10,7 +10,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -40,7 +40,7 @@
 \*****************************************************************************/
 
 #include "scontrol.h"
-#include "src/plugins/select/bluegene/wrap_rm_api.h"
+#include "src/plugins/select/bluegene/bg_enums.h"
 #include "src/common/proc_args.h"
 
 #define OPT_LONG_HIDE   0x102
@@ -55,8 +55,10 @@ int input_words;	/* number of words of input permitted */
 int one_liner;		/* one record per line if =1 */
 int quiet_flag;		/* quiet=1, verbose=-1, normal=0 */
 int verbosity;		/* count of "-v" options */
-uint32_t cluster_flags; /*what type of cluster are we talking to */
+uint32_t cluster_flags; /* what type of cluster are we talking to */
+
 block_info_msg_t *old_block_info_ptr = NULL;
+front_end_info_msg_t *old_front_end_info_ptr = NULL;
 job_info_msg_t *old_job_info_ptr = NULL;
 node_info_msg_t *old_node_info_ptr = NULL;
 partition_info_msg_t *old_part_info_ptr = NULL;
@@ -86,7 +88,7 @@ int
 main (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS, i, opt_char, input_field_count;
-	char **input_fields;
+	char **input_fields, *env_val;
 	log_options_t opts = LOG_OPTS_STDERR_ONLY ;
 
 	int option_index;
@@ -117,6 +119,17 @@ main (int argc, char *argv[])
 
 	if (getenv ("SCONTROL_ALL"))
 		all_flag= 1;
+	if ((env_val = getenv("SLURM_CLUSTERS"))) {
+		if (!(clusters = slurmdb_get_info_cluster(env_val))) {
+			error("'%s' can't be reached now, "
+			      "or it is an invalid entry for "
+			      "SLURM_CLUSTERS.  Use 'sacctmgr --list "
+			      "cluster' to see available clusters.",
+			      env_val);
+			exit(1);
+		}
+		working_cluster_rec = list_peek(clusters);
+	}
 
 	while((opt_char = getopt_long(argc, argv, "adhM:oQvV",
 			long_options, &option_index)) != -1) {
@@ -141,13 +154,16 @@ main (int argc, char *argv[])
 			detail_flag = 0;
 			break;
 		case (int)'M':
-			if(clusters) {
+			if (clusters) {
 				list_destroy(clusters);
 				clusters = NULL;
 				working_cluster_rec = NULL;
 			}
-			if(!(clusters = slurmdb_get_info_cluster(optarg))) {
-				error("'%s' invalid entry for --cluster",
+			if (!(clusters = slurmdb_get_info_cluster(optarg))) {
+				error("'%s' can't be reached now, "
+				      "or it is an invalid entry for "
+				      "--cluster.  Use 'sacctmgr --list "
+				      "cluster' to see available clusters.",
 				      optarg);
 				exit(1);
 			}
@@ -175,6 +191,8 @@ main (int argc, char *argv[])
 		}
 	}
 
+	if (clusters && (list_count(clusters) > 1))
+		fatal("Only one cluster can be used at a time with scontrol");
 	cluster_flags = slurmdb_setup_cluster_flags();
 
 	if (verbosity) {
@@ -205,7 +223,7 @@ main (int argc, char *argv[])
 			break;
 		error_code = _get_command (&input_field_count, input_fields);
 	}
-	if(clusters)
+	if (clusters)
 		list_destroy(clusters);
 	exit(exit_code);
 }
@@ -536,7 +554,7 @@ _process_command (int argc, char *argv[])
 {
 	int error_code = 0;
 	char *tag = argv[0];
-	int taglen = 0;
+	int tag_len = 0;
 
 	if (argc < 1) {
 		exit_code = 1;
@@ -544,14 +562,14 @@ _process_command (int argc, char *argv[])
 			fprintf(stderr, "no input");
 		return 0;
 	} else if(tag)
-		taglen = strlen(tag);
+		tag_len = strlen(tag);
 	else {
 		if (quiet_flag == -1)
 			fprintf(stderr, "input problem");
 		return 0;
 	}
 
-	if (strncasecmp (tag, "abort", MAX(taglen, 5)) == 0) {
+	if (strncasecmp (tag, "abort", MAX(tag_len, 5)) == 0) {
 		/* require full command name */
 		if (argc > 2) {
 			exit_code = 1;
@@ -566,9 +584,9 @@ _process_command (int argc, char *argv[])
 				slurm_perror ("slurm_shutdown error");
 		}
 	}
-	else if (strncasecmp (tag, "all", MAX(taglen, 2)) == 0)
+	else if (strncasecmp (tag, "all", MAX(tag_len, 2)) == 0)
 		all_flag = 1;
-	else if (strncasecmp (tag, "completing", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "completing", MAX(tag_len, 2)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -577,23 +595,32 @@ _process_command (int argc, char *argv[])
 		}
 		scontrol_print_completing();
 	}
-	else if (strncasecmp (tag, "cluster", MAX(taglen, 2)) == 0) {
-		if(clusters) {
+	else if (strncasecmp (tag, "cluster", MAX(tag_len, 2)) == 0) {
+		if (clusters) {
 			list_destroy(clusters);
 			clusters = NULL;
 			working_cluster_rec = NULL;
 		}
 		if (argc >= 2) {
-			if(!(clusters = slurmdb_get_info_cluster(argv[1]))) {
-				error("'%s' invalid entry for --cluster",
+			if (!(clusters = slurmdb_get_info_cluster(argv[1]))) {
+				error("'%s' can't be reached now, "
+				      "or it is an invalid entry for "
+				      "--cluster.  Use 'sacctmgr --list "
+				      "cluster' to see available clusters.",
 				      optarg);
 				exit(1);
 			}
 			working_cluster_rec = list_peek(clusters);
+			if (list_count(clusters) > 1) {
+				fatal("Only one cluster can be used at a time "
+				      "with scontrol");
+			}
 		}
 		cluster_flags = slurmdb_setup_cluster_flags();
 		slurm_free_block_info_msg(old_block_info_ptr);
 		old_block_info_ptr = NULL;
+		slurm_free_front_end_info_msg(old_front_end_info_ptr);
+		old_front_end_info_ptr = NULL;
 		slurm_free_job_info_msg(old_job_info_ptr);
 		old_job_info_ptr = NULL;
 		slurm_free_node_info_msg(old_node_info_ptr);
@@ -617,7 +644,7 @@ _process_command (int argc, char *argv[])
 		/* if(old_slurm_ctl_conf_ptr) */
 		/* 	old_slurm_ctl_conf_ptr->last_update = 0; */
 	}
-	else if (strncasecmp (tag, "create", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "create", MAX(tag_len, 2)) == 0) {
 		if (argc < 2) {
 			exit_code = 1;
 			fprintf (stderr, "too few arguments for %s keyword\n",
@@ -626,7 +653,7 @@ _process_command (int argc, char *argv[])
 		}
 		_create_it ((argc - 1), &argv[1]);
 	}
-	else if (strncasecmp (tag, "details", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "details", MAX(tag_len, 1)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -636,7 +663,7 @@ _process_command (int argc, char *argv[])
 		}
 		detail_flag = 1;
 	}
-	else if (strncasecmp (tag, "exit", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "exit", MAX(tag_len, 1)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -645,7 +672,7 @@ _process_command (int argc, char *argv[])
 		}
 		exit_flag = 1;
 	}
-	else if (strncasecmp (tag, "help", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "help", MAX(tag_len, 2)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -654,11 +681,11 @@ _process_command (int argc, char *argv[])
 		}
 		_usage ();
 	}
-	else if (strncasecmp (tag, "hide", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "hide", MAX(tag_len, 2)) == 0) {
 		all_flag = 0;
 		detail_flag = 0;
 	}
-	else if (strncasecmp (tag, "oneliner", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "oneliner", MAX(tag_len, 1)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -667,7 +694,7 @@ _process_command (int argc, char *argv[])
 		}
 		one_liner = 1;
 	}
-	else if (strncasecmp (tag, "pidinfo", MAX(taglen, 3)) == 0) {
+	else if (strncasecmp (tag, "pidinfo", MAX(tag_len, 3)) == 0) {
 		if (argc > 2) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -681,7 +708,7 @@ _process_command (int argc, char *argv[])
 		} else
 			scontrol_pid_info ((pid_t) atol (argv[1]) );
 	}
-	else if (strncasecmp (tag, "ping", MAX(taglen, 3)) == 0) {
+	else if (strncasecmp (tag, "ping", MAX(tag_len, 3)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -691,7 +718,7 @@ _process_command (int argc, char *argv[])
 		_print_ping ();
 	}
 	else if ((strncasecmp (tag, "\\q", 2) == 0) ||
-		 (strncasecmp (tag, "quiet", MAX(taglen, 4)) == 0)) {
+		 (strncasecmp (tag, "quiet", MAX(tag_len, 4)) == 0)) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr, "too many arguments for keyword:%s\n",
@@ -699,7 +726,7 @@ _process_command (int argc, char *argv[])
 		}
 		quiet_flag = 1;
 	}
-	else if (strncasecmp (tag, "quit", MAX(taglen, 4)) == 0) {
+	else if (strncasecmp (tag, "quit", MAX(tag_len, 4)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -708,7 +735,7 @@ _process_command (int argc, char *argv[])
 		}
 		exit_flag = 1;
 	}
-	else if (strncasecmp (tag, "reconfigure", MAX(taglen, 3)) == 0) {
+	else if (strncasecmp (tag, "reconfigure", MAX(tag_len, 3)) == 0) {
 		if (argc > 2) {
 			exit_code = 1;
 			fprintf (stderr, "too many arguments for keyword:%s\n",
@@ -721,7 +748,7 @@ _process_command (int argc, char *argv[])
 				slurm_perror ("slurm_reconfigure error");
 		}
 	}
-	else if (strncasecmp (tag, "checkpoint", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "checkpoint", MAX(tag_len, 2)) == 0) {
 		if (argc > 5) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -747,7 +774,7 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if (strncasecmp (tag, "requeue", MAX(taglen, 3)) == 0) {
+	else if (strncasecmp (tag, "requeue", MAX(tag_len, 3)) == 0) {
 		if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -773,7 +800,7 @@ _process_command (int argc, char *argv[])
 	else if ((strncasecmp (tag, "hold",  4) == 0) ||
 		 (strncasecmp (tag, "holdu", 5) == 0) ||
 		 (strncasecmp (tag, "uhold", 5) == 0) ||
-		 (strncasecmp (tag, "release", MAX(taglen, 3)) == 0)) {
+	         (strncasecmp (tag, "release", MAX(tag_len, 3)) == 0)) {
 		if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -796,8 +823,8 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if ((strncasecmp (tag, "suspend", MAX(taglen, 2)) == 0) ||
-		 (strncasecmp (tag, "resume", MAX(taglen, 3)) == 0)) {
+	else if ((strncasecmp (tag, "suspend", MAX(tag_len, 2)) == 0) ||
+	         (strncasecmp (tag, "resume", MAX(tag_len, 3)) == 0)) {
 		if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -820,15 +847,17 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if (strncasecmp (tag, "wait_job", MAX(taglen, 2)) == 0) {
-		if (argc > 2) {
+	else if (strncasecmp (tag, "wait_job", MAX(tag_len, 2)) == 0) {
+		if (cluster_flags & CLUSTER_FLAG_CRAYXT) {
+			fprintf(stderr,
+				"wait_job is handled automatically on Cray.\n");
+		} else if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
 				fprintf(stderr,
 					"too many arguments for keyword:%s\n",
 					tag);
-		}
-		else if (argc < 2) {
+		} else if (argc < 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
 				fprintf(stderr,
@@ -840,7 +869,65 @@ _process_command (int argc, char *argv[])
 				exit_code = 1;
 		}
 	}
-	else if (strncasecmp (tag, "setdebug", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "setdebugflags", MAX(tag_len, 9)) == 0) {
+		if (argc > 2) {
+			exit_code = 1;
+			if (quiet_flag != 1)
+				fprintf(stderr,
+					"too many arguments for keyword:%s\n",
+					tag);
+		} else if (argc < 2) {
+			exit_code = 1;
+			if (quiet_flag != 1)
+				fprintf(stderr,
+					"too few arguments for keyword:%s\n",
+					tag);
+		} else {
+			int i, mode = 0;
+			uint32_t debug_flags_plus  = 0;
+			uint32_t debug_flags_minus = 0, flags;
+
+			for (i = 1; i < argc; i++) {
+				if (argv[i][0] == '+')
+					mode = 1;
+				else if (argv[i][0] == '-')
+					mode = -1;
+				else {
+					mode = 0;
+					break;
+				}
+				flags = debug_str2flags(&argv[i][1]);
+				if (flags == NO_VAL)
+					break;
+				if (mode == 1)
+					debug_flags_plus  |= flags;
+				else
+					debug_flags_minus |= flags;
+			}
+			if (i < argc) {
+				exit_code = 1;
+				if (quiet_flag != 1) {
+					fprintf(stderr, "invalid debug "
+						"flag: %s\n", argv[i]);
+				}
+				if ((quiet_flag != 1) &&  (mode = 0)) {
+					fprintf(stderr, "Usage: setdebugflags"
+						" [+|-]NAME\n");
+				}
+			} else {
+				error_code = slurm_set_debugflags(
+					debug_flags_plus, debug_flags_minus);
+				if (error_code) {
+					exit_code = 1;
+					if (quiet_flag != 1)
+						slurm_perror(
+							"slurm_set_debug_flags"
+							" error");
+				}
+			}
+		}
+	}
+	else if (strncasecmp (tag, "setdebug", MAX(tag_len, 2)) == 0) {
 		if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -893,7 +980,7 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if (strncasecmp (tag, "schedloglevel", MAX(taglen, 2)) == 0) {
+	else if (strncasecmp (tag, "schedloglevel", MAX(tag_len, 2)) == 0) {
 		if (argc > 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -943,10 +1030,10 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if (strncasecmp (tag, "show", MAX(taglen, 3)) == 0) {
+	else if (strncasecmp (tag, "show", MAX(tag_len, 3)) == 0) {
 		_show_it (argc, argv);
 	}
-	else if (strncasecmp (tag, "takeover", MAX(taglen, 8)) == 0) {
+	else if (strncasecmp (tag, "takeover", MAX(tag_len, 8)) == 0) {
 		char *secondary = NULL;
 		slurm_ctl_conf_info_msg_t  *slurm_ctl_conf_ptr = NULL;
 
@@ -967,7 +1054,7 @@ _process_command (int argc, char *argv[])
 		}
 		xfree(secondary);
 	}
-	else if (strncasecmp (tag, "shutdown", MAX(taglen, 8)) == 0) {
+	else if (strncasecmp (tag, "shutdown", MAX(tag_len, 8)) == 0) {
 		/* require full command name */
 		uint16_t options = 0;
 		if (argc == 2) {
@@ -996,7 +1083,7 @@ _process_command (int argc, char *argv[])
 			}
 		}
 	}
-	else if (strncasecmp (tag, "update", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "update", MAX(tag_len, 1)) == 0) {
 		if (argc < 2) {
 			exit_code = 1;
 			fprintf (stderr, "too few arguments for %s keyword\n",
@@ -1005,7 +1092,7 @@ _process_command (int argc, char *argv[])
 		}
 		_update_it ((argc - 1), &argv[1]);
 	}
-	else if (strncasecmp (tag, "delete", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "delete", MAX(tag_len, 1)) == 0) {
 		if (argc < 2) {
 			exit_code = 1;
 			fprintf (stderr, "too few arguments for %s keyword\n",
@@ -1014,7 +1101,7 @@ _process_command (int argc, char *argv[])
 		}
 		_delete_it ((argc - 1), &argv[1]);
 	}
-	else if (strncasecmp (tag, "verbose", MAX(taglen, 4)) == 0) {
+	else if (strncasecmp (tag, "verbose", MAX(tag_len, 4)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -1023,7 +1110,7 @@ _process_command (int argc, char *argv[])
 		}
 		quiet_flag = -1;
 	}
-	else if (strncasecmp (tag, "version", MAX(taglen, 4)) == 0) {
+	else if (strncasecmp (tag, "version", MAX(tag_len, 4)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -1032,7 +1119,7 @@ _process_command (int argc, char *argv[])
 		}
 		_print_version();
 	}
-	else if (strncasecmp (tag, "listpids", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "listpids", MAX(tag_len, 1)) == 0) {
 		if (argc > 3) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -1043,7 +1130,7 @@ _process_command (int argc, char *argv[])
 					    argc <= 2 ? NULL : argv[2]);
 		}
 	}
-	else if (strncasecmp (tag, "notify", MAX(taglen, 1)) == 0) {
+	else if (strncasecmp (tag, "notify", MAX(tag_len, 1)) == 0) {
 		if (argc < 3) {
 			exit_code = 1;
 			fprintf (stderr,
@@ -1078,18 +1165,18 @@ _create_it (int argc, char *argv[])
 	for (i=0; i<argc; i++) {
 		char *tag = argv[i];
 		char *val = strchr(argv[i], '=');
-		int taglen;
+		int tag_len;
 
 		if (val) {
-			taglen = val - argv[i];
+			tag_len = val - argv[i];
 			val++;
 		} else {
-			taglen = strlen(tag);
+			tag_len = strlen(tag);
 		}
-		if (!strncasecmp(tag, "ReservationName", MAX(taglen, 3))) {
+		if (!strncasecmp(tag, "ReservationName", MAX(tag_len, 3))) {
 			error_code = scontrol_create_res(argc, argv);
 			break;
-		} else if (!strncasecmp(tag, "PartitionName", MAX(taglen, 3))) {
+		} else if (!strncasecmp(tag, "PartitionName", MAX(tag_len, 3))) {
 			error_code = scontrol_create_part(argc, argv);
 			break;
 		}
@@ -1114,7 +1201,7 @@ static void
 _delete_it (int argc, char *argv[])
 {
 	char *tag = NULL, *val = NULL;
-	int taglen = 0;
+	int tag_len = 0;
 
 	if (argc != 1) {
 		error("Only one option follows delete.  %d given.", argc);
@@ -1125,7 +1212,7 @@ _delete_it (int argc, char *argv[])
 	tag = argv[0];
 	val = strchr(argv[0], '=');
 	if (val) {
-		taglen = val - argv[0];
+		tag_len = val - argv[0];
 		val++;
 	} else {
 		error("Proper format is 'delete Partition=p'"
@@ -1135,7 +1222,7 @@ _delete_it (int argc, char *argv[])
 	}
 
 	/* First identify the entity type to delete */
-	if (strncasecmp (tag, "PartitionName", MAX(taglen, 3)) == 0) {
+	if (strncasecmp (tag, "PartitionName", MAX(tag_len, 3)) == 0) {
 		delete_part_msg_t part_msg;
 		part_msg.name = val;
 		if (slurm_delete_partition(&part_msg)) {
@@ -1143,7 +1230,7 @@ _delete_it (int argc, char *argv[])
 			snprintf(errmsg, 64, "delete_partition %s", argv[0]);
 			slurm_perror(errmsg);
 		}
-	} else if (strncasecmp (tag, "ReservationName", MAX(taglen, 3)) == 0) {
+	} else if (strncasecmp (tag, "ReservationName", MAX(tag_len, 3)) == 0) {
 		reservation_name_msg_t   res_msg;
 		res_msg.name = val;
 		if (slurm_delete_reservation(&res_msg)) {
@@ -1151,12 +1238,12 @@ _delete_it (int argc, char *argv[])
 			snprintf(errmsg, 64, "delete_reservation %s", argv[0]);
 			slurm_perror(errmsg);
 		}
-	} else if (strncasecmp (tag, "BlockName", MAX(taglen, 3)) == 0) {
+	} else if (strncasecmp (tag, "BlockName", MAX(tag_len, 3)) == 0) {
 		if(cluster_flags & CLUSTER_FLAG_BG) {
 			update_block_msg_t   block_msg;
 			slurm_init_update_block_msg ( &block_msg );
 			block_msg.bg_block_id = val;
-			block_msg.state = RM_PARTITION_NAV;
+			block_msg.state = BG_BLOCK_NAV;
 			if (slurm_update_block(&block_msg)) {
 				char errmsg[64];
 				snprintf(errmsg, 64, "delete_block %s",
@@ -1184,7 +1271,7 @@ static void
 _show_it (int argc, char *argv[])
 {
 	char *tag = NULL, *val = NULL;
-	int taglen = 0;
+	int tag_len = 0;
 
 	if (argc > 3) {
 		exit_code = 1;
@@ -1203,10 +1290,10 @@ _show_it (int argc, char *argv[])
 	}
 
 	tag = argv[1];
-	taglen = strlen(tag);
+	tag_len = strlen(tag);
 	val = strchr(argv[1], '=');
 	if (val) {
-		taglen = val - argv[1];
+		tag_len = val - argv[1];
 		val++;
 	} else if (argc == 3) {
 		val = argv[2];
@@ -1214,11 +1301,16 @@ _show_it (int argc, char *argv[])
 		val = NULL;
 	}
 
-	if (strncasecmp (tag, "blocks", MAX(taglen, 1)) == 0) {
+	if (strncasecmp (tag, "aliases", MAX(tag_len, 1)) == 0) {
+		if (val)
+			_print_aliases (val);
+		else
+			_print_aliases (NULL);
+	} else if (strncasecmp (tag, "blocks", MAX(tag_len, 1)) == 0) {
 		scontrol_print_block (val);
-	} else if (strncasecmp (tag, "config", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "config", MAX(tag_len, 1)) == 0) {
 		_print_config (val);
-	} else if (strncasecmp (tag, "daemons", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "daemons", MAX(tag_len, 1)) == 0) {
 		if (val) {
 			exit_code = 1;
 			if (quiet_flag != 1)
@@ -1227,39 +1319,36 @@ _show_it (int argc, char *argv[])
 					argv[0]);
 		}
 		_print_daemons ();
-	} else if (strncasecmp (tag, "aliases", MAX(taglen, 1)) == 0) {
-		if (val)
-			_print_aliases (val);
-		else
-			_print_aliases (NULL);
-	} else if (strncasecmp (tag, "jobs", MAX(taglen, 1)) == 0 ||
-		   strncasecmp (tag, "jobid", MAX(taglen, 1)) == 0 ) {
-		scontrol_print_job (val);
-	} else if (strncasecmp (tag, "hostnames", MAX(taglen, 5)) == 0) {
+	} else if (strncasecmp (tag, "FrontendName",  MAX(tag_len, 1)) == 0) {
+		scontrol_print_front_end_list(val);
+	} else if (strncasecmp (tag, "hostnames", MAX(tag_len, 5)) == 0) {
 		if (val)
 			scontrol_print_hosts(val);
 		else
 			scontrol_print_hosts(getenv("SLURM_NODELIST"));
-	} else if (strncasecmp (tag, "hostlist", MAX(taglen, 5)) == 0) {
+	} else if (strncasecmp (tag, "hostlist", MAX(tag_len, 5)) == 0) {
 		if (!val) {
 			exit_code = 1;
 			fprintf(stderr, "invalid encode argument\n");
 			_usage();
 		} else if (scontrol_encode_hostlist(val))
 			exit_code = 1;
-	} else if (strncasecmp (tag, "nodes", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "jobs", MAX(tag_len, 1)) == 0 ||
+		   strncasecmp (tag, "jobid", MAX(tag_len, 1)) == 0 ) {
+		scontrol_print_job (val);
+	} else if (strncasecmp (tag, "nodes", MAX(tag_len, 1)) == 0) {
 		scontrol_print_node_list (val);
-	} else if (strncasecmp (tag, "partitions", MAX(taglen, 1)) == 0 ||
-		   strncasecmp (tag, "partitionname", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "partitions", MAX(tag_len, 1)) == 0 ||
+		   strncasecmp (tag, "partitionname", MAX(tag_len, 1)) == 0) {
 		scontrol_print_part (val);
-	} else if (strncasecmp (tag, "reservations", MAX(taglen, 1)) == 0 ||
-		   strncasecmp (tag, "reservationname", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "reservations", MAX(tag_len, 1)) == 0 ||
+		   strncasecmp (tag, "reservationname", MAX(tag_len, 1)) == 0) {
 		scontrol_print_res (val);
-	} else if (strncasecmp (tag, "slurmd", MAX(taglen, 2)) == 0) {
+	} else if (strncasecmp (tag, "slurmd", MAX(tag_len, 2)) == 0) {
 		_print_slurmd (val);
-	} else if (strncasecmp (tag, "steps", MAX(taglen, 2)) == 0) {
+	} else if (strncasecmp (tag, "steps", MAX(tag_len, 2)) == 0) {
 		scontrol_print_step (val);
-	} else if (strncasecmp (tag, "topology", MAX(taglen, 1)) == 0) {
+	} else if (strncasecmp (tag, "topology", MAX(tag_len, 1)) == 0) {
 		scontrol_print_topo (val);
 	} else {
 		exit_code = 1;
@@ -1283,39 +1372,43 @@ _update_it (int argc, char *argv[])
 {
 	char *val = NULL;
 	int i, error_code = SLURM_SUCCESS;
-	int nodetag=0, partag=0, jobtag=0;
-	int blocktag=0, subtag=0, restag=0;
-	int debugtag=0, steptag=0;
+	int node_tag = 0, part_tag = 0, job_tag = 0;
+	int block_tag = 0, sub_tag = 0, res_tag = 0;
+	int debug_tag = 0, step_tag = 0, front_end_tag = 0;
 
 	/* First identify the entity to update */
 	for (i=0; i<argc; i++) {
 		char *tag = argv[i];
-		int taglen = 0;
+		int tag_len = 0;
 		val = strchr(argv[i], '=');
 		if (!val)
 			continue;
-		taglen = val - argv[i];
+		tag_len = val - argv[i];
 		val++;
 
-		if (!strncasecmp(tag, "NodeName", MAX(taglen, 3))) {
-			nodetag=1;
-		} else if (!strncasecmp(tag, "PartitionName", MAX(taglen, 3))) {
-			partag=1;
-		} else if (!strncasecmp(tag, "JobId", MAX(taglen, 3))) {
-			jobtag=1;
-		} else if (!strncasecmp(tag, "StepId", MAX(taglen, 4))) {
-			steptag=1;
-		} else if (!strncasecmp(tag, "BlockName", MAX(taglen, 3))) {
-			blocktag=1;
-		} else if (!strncasecmp(tag, "SubBPName", MAX(taglen, 3))) {
-			subtag=1;
+		if (!strncasecmp(tag, "NodeName", MAX(tag_len, 3))) {
+			node_tag = 1;
+		} else if (!strncasecmp(tag, "PartitionName",
+					MAX(tag_len, 3))) {
+			part_tag = 1;
+		} else if (!strncasecmp(tag, "JobId", MAX(tag_len, 3))) {
+			job_tag = 1;
+		} else if (!strncasecmp(tag, "StepId", MAX(tag_len, 4))) {
+			step_tag = 1;
+		} else if (!strncasecmp(tag, "BlockName", MAX(tag_len, 3))) {
+			block_tag = 1;
+		} else if (!strncasecmp(tag, "SubBPName", MAX(tag_len, 3))) {
+			sub_tag = 1;
+		} else if (!strncasecmp(tag, "FrontendName",
+					MAX(tag_len, 2))) {
+			front_end_tag = 1;
 		} else if (!strncasecmp(tag, "ReservationName",
-					MAX(taglen, 3))) {
-			restag=1;
+					MAX(tag_len, 3))) {
+			res_tag = 1;
 		} else if (!strncasecmp(tag, "SlurmctldDebug",
-					MAX(taglen, 2))) {
-			debugtag=1;
-		}
+					MAX(tag_len, 2))) {
+			debug_tag= 1;
+		} 
 	}
 
 	/* The order of tests matters here.  An update job request can include
@@ -1325,21 +1418,23 @@ _update_it (int argc, char *argv[])
 	 * partition tag.  The order of the rest doesn't matter because there
 	 * aren't any other duplicate tags.  */
 
-	if (jobtag)
+	if (job_tag)
 		error_code = scontrol_update_job (argc, argv);
-	else if (steptag)
+	else if (step_tag)
 		error_code = scontrol_update_step (argc, argv);
-	else if (restag)
+	else if (res_tag)
 		error_code = scontrol_update_res (argc, argv);
-	else if (nodetag)
+	else if (node_tag)
 		error_code = scontrol_update_node (argc, argv);
-	else if (partag)
+	else if (front_end_tag)
+		error_code = scontrol_update_front_end (argc, argv);
+	else if (part_tag)
 		error_code = scontrol_update_part (argc, argv);
-	else if (blocktag)
+	else if (block_tag)
 		error_code = _update_bluegene_block (argc, argv);
-	else if (subtag)
+	else if (sub_tag)
 		error_code = _update_bluegene_subbp (argc, argv);
-	else if (debugtag)
+	else if (debug_tag)
 		error_code = _update_slurmctld_debug(val);
 	else {
 		exit_code = 1;
@@ -1384,10 +1479,10 @@ _update_bluegene_block (int argc, char *argv[])
 	for (i=0; i<argc; i++) {
 		char *tag = argv[i];
 		char *val = strchr(argv[i], '=');
-		int taglen = 0, vallen = 0;
+		int tag_len = 0, vallen = 0;
 
 		if (val) {
-			taglen = val - argv[i];
+			tag_len = val - argv[i];
 			val++;
 			vallen = strlen(val);
 		} else {
@@ -1398,19 +1493,19 @@ _update_bluegene_block (int argc, char *argv[])
 			return 0;
 		}
 
-		if (!strncasecmp(tag, "BlockName", MAX(taglen, 2))) {
+		if (!strncasecmp(tag, "BlockName", MAX(tag_len, 2))) {
 			block_msg.bg_block_id = val;
-		} else if (!strncasecmp(tag, "State", MAX(taglen, 2))) {
+		} else if (!strncasecmp(tag, "State", MAX(tag_len, 2))) {
 			if (!strncasecmp(val, "ERROR", MAX(vallen, 1)))
-				block_msg.state = RM_PARTITION_ERROR;
+				block_msg.state = BG_BLOCK_ERROR_FLAG;
 			else if (!strncasecmp(val, "FREE", MAX(vallen, 1)))
-				block_msg.state = RM_PARTITION_FREE;
+				block_msg.state = BG_BLOCK_FREE;
 			else if (!strncasecmp(val, "RECREATE", MAX(vallen, 3)))
-				block_msg.state = RM_PARTITION_CONFIGURING;
+				block_msg.state = BG_BLOCK_BOOTING;
 			else if (!strncasecmp(val, "REMOVE", MAX(vallen, 3)))
-				block_msg.state = RM_PARTITION_NAV;
+				block_msg.state = BG_BLOCK_NAV;
 			else if (!strncasecmp(val, "RESUME", MAX(vallen, 3)))
-				block_msg.state = RM_PARTITION_DEALLOCATING;
+				block_msg.state = BG_BLOCK_TERM;
 			else {
 				exit_code = 1;
 				fprintf (stderr, "Invalid input: %s\n",
@@ -1435,7 +1530,7 @@ _update_bluegene_block (int argc, char *argv[])
 		return 0;
 	} else if (block_msg.state == (uint16_t)NO_VAL) {
 		error("You didn't give me a state to set %s to "
-		      "(i.e. FREE, ERROR).", block_msg.nodes);
+		      "(i.e. FREE, ERROR).", block_msg.mp_str);
 		return 0;
 	}
 
@@ -1471,10 +1566,10 @@ _update_bluegene_subbp (int argc, char *argv[])
 	for (i=0; i<argc; i++) {
 		char *tag = argv[i];
 		char *val = strchr(argv[i], '=');
-		int taglen = 0, vallen = 0;
+		int tag_len = 0, vallen = 0;
 
 		if (val) {
-			taglen = val - argv[i];
+			tag_len = val - argv[i];
 			val++;
 			vallen = strlen(val);
 		} else {
@@ -1484,13 +1579,13 @@ _update_bluegene_subbp (int argc, char *argv[])
 			return 0;
 		}
 
-		if (!strncasecmp(tag, "SubBPName", MAX(taglen, 2)))
-			block_msg.nodes = val;
-		else if (!strncasecmp(tag, "State", MAX(taglen, 2))) {
+		if (!strncasecmp(tag, "SubBPName", MAX(tag_len, 2)))
+			block_msg.mp_str = val;
+		else if (!strncasecmp(tag, "State", MAX(tag_len, 2))) {
 			if (!strncasecmp(val, "ERROR", MAX(vallen, 1)))
-				block_msg.state = RM_PARTITION_ERROR;
+				block_msg.state = BG_BLOCK_ERROR_FLAG;
 			else if (!strncasecmp(val, "FREE", MAX(vallen, 1)))
-				block_msg.state = RM_PARTITION_FREE;
+				block_msg.state = BG_BLOCK_FREE;
 			else {
 				exit_code = 1;
 				fprintf (stderr, "Invalid input: %s\n",
@@ -1508,12 +1603,12 @@ _update_bluegene_subbp (int argc, char *argv[])
 		}
 	}
 
-	if(!block_msg.nodes) {
+	if(!block_msg.mp_str) {
 		error("You didn't supply an ionode list.");
 		return 0;
 	} else if (block_msg.state == (uint16_t)NO_VAL) {
 		error("You didn't give me a state to set %s to "
-		      "(i.e. FREE, ERROR).", block_msg.nodes);
+		      "(i.e. FREE, ERROR).", block_msg.mp_str);
 		return 0;
 	}
 
@@ -1609,6 +1704,7 @@ scontrol [<OPTION>] [<COMMAND>]                                            \n\
      requeue <job_id>         re-queue a batch job                         \n\
      resume <job_id>          resume previously suspended job (see suspend)\n\
      setdebug <level>         set slurmctld debug level                    \n\
+     setdebugflags [+|-]<flag>  add or remove slurmctld DebugFlags         \n\
      schedloglevel <slevel>   set scheduler log level                      \n\
      show <ENTITY> [<ID>]     display state of identified entity, default  \n\
 			      is all records.                              \n\
@@ -1625,9 +1721,9 @@ scontrol [<OPTION>] [<COMMAND>]                                            \n\
 			      are booted and usable                        \n\
      !!                       Repeat the last command entered.             \n\
 									   \n\
-  <ENTITY> may be \"aliases\", \"config\", \"daemons\", \"job\",           \n\
-       \"node\", \"partition\", \"reservation\", \"hostlist\",             \n\
-       \"hostnames\", \"slurmd\", \"topology\", or \"step\"                \n\
+  <ENTITY> may be \"aliases\", \"config\", \"daemons\", \"frontend\",      \n\
+       \"hostlist\", \"hostnames\", \"job\", \"node\", \"partition\",      \n\
+       \"reservation\", \"slurmd\", \"step\", or \"topology\"              \n\
        (also for BlueGene only: \"block\" or \"subbp\").                   \n\
 									   \n\
   <ID> may be a configuration parameter name, job id, node name, partition \n\
