@@ -8,7 +8,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -189,6 +189,59 @@ static int _slurm_preempt_context_destroy(slurm_preempt_context_t *c)
 	return SLURM_SUCCESS;
 }
 
+/* *********************************************************************** */
+/*  TAG(                    _preempt_signal                             )  */
+/* *********************************************************************** */
+static void _preempt_signal(struct job_record *job_ptr, uint32_t grace_time)
+{
+	/* allow the job termination mechanism to signal the job */
+
+	job_ptr->preempt_time = time(NULL);
+	job_ptr->end_time = job_ptr->preempt_time + (time_t)grace_time;
+}
+/* *********************************************************************** */
+/*  TAG(                    slurm_job_check_grace                       )  */
+/* *********************************************************************** */
+extern int slurm_job_check_grace(struct job_record *job_ptr)
+{
+	/* Preempt modes: -1 (unset), 0 (none), 1 (partition), 2 (QOS) */
+	static int preempt_mode = 0;
+	static time_t last_update_time = 0;
+	int rc = SLURM_SUCCESS;
+	uint32_t grace_time = 0;
+
+	if (job_ptr->preempt_time) {
+		if (time(NULL) >= job_ptr->end_time)
+			rc = SLURM_ERROR;
+		return rc;
+	}
+
+	if (last_update_time != slurmctld_conf.last_update) {
+		char *preempt_type = slurm_get_preempt_type();
+		if ((strcmp(preempt_type, "preempt/partition_prio") == 0))
+			preempt_mode = 1;
+		else if ((strcmp(preempt_type, "preempt/qos") == 0))
+			preempt_mode = 2;
+		else
+			preempt_mode = 0;
+		xfree(preempt_type);
+	}
+
+	if (preempt_mode == 1)
+		grace_time = job_ptr->part_ptr->grace_time;
+	else if (preempt_mode == 2) {
+		slurmdb_qos_rec_t *qos_ptr = (slurmdb_qos_rec_t *)
+					     job_ptr->qos_ptr;
+		grace_time = qos_ptr->grace_time;
+	}
+
+	if (grace_time)
+		_preempt_signal(job_ptr, grace_time);
+	else
+		rc = SLURM_ERROR;
+
+	return rc;
+}
 
 /* *********************************************************************** */
 /*  TAG(                    slurm_preempt_init                        )  */
