@@ -8,7 +8,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -44,7 +44,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <slurm/slurm_errno.h>
+
+#include "slurm/slurm_errno.h"
 
 #include "src/common/log.h"
 #include "src/common/list.h"
@@ -112,6 +113,7 @@ static void _clear_slurmdbd_conf(void)
 		xfree(slurmdbd_conf->storage_type);
 		xfree(slurmdbd_conf->storage_user);
 		slurmdbd_conf->track_wckey = 0;
+		slurmdbd_conf->track_ctld = 0;
 	}
 }
 
@@ -162,6 +164,7 @@ extern int read_slurmdbd_conf(void)
 		{"StorageType", S_P_STRING},
 		{"StorageUser", S_P_STRING},
 		{"TrackWCKey", S_P_BOOLEAN},
+		{"TrackSlurmctldDown", S_P_BOOLEAN},
 		{NULL} };
 	s_p_hashtbl_t *tbl = NULL;
 	char *conf_path = NULL;
@@ -186,7 +189,8 @@ extern int read_slurmdbd_conf(void)
 		debug("Reading slurmdbd.conf file %s", conf_path);
 
 		tbl = s_p_hashtbl_create(options);
-		if (s_p_parse_file(tbl, NULL, conf_path) == SLURM_ERROR) {
+		if (s_p_parse_file(tbl, NULL, conf_path, false)
+		    == SLURM_ERROR) {
 			fatal("Could not open/read/parse slurmdbd.conf file %s",
 			      conf_path);
 		}
@@ -358,6 +362,10 @@ extern int read_slurmdbd_conf(void)
 				     "TrackWCKey", tbl))
 			slurmdbd_conf->track_wckey = false;
 
+		if (!s_p_get_boolean((bool *)&slurmdbd_conf->track_ctld,
+				     "TrackSlurmctldDown", tbl))
+			slurmdbd_conf->track_ctld = false;
+
 		if (a_events)
 			slurmdbd_conf->purge_event |= SLURMDB_PURGE_ARCHIVE;
 		if (a_jobs)
@@ -536,6 +544,7 @@ extern void log_config(void)
 	debug2("StorageUser       = %s", slurmdbd_conf->storage_user);
 
 	debug2("TrackWCKey        = %u", slurmdbd_conf->track_wckey);
+	debug2("TrackSlurmctldDown= %u", slurmdbd_conf->track_ctld);
 }
 
 /* Return the DbdPort value */
@@ -805,5 +814,11 @@ extern List dump_config(void)
 	snprintf(key_pair->value, 32, "%u", slurmdbd_conf->track_wckey);
 	list_append(my_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("TrackSlurmctldDown");
+	key_pair->value = xmalloc(32);
+	snprintf(key_pair->value, 32, "%u", slurmdbd_conf->track_ctld);
+	list_append(my_list, key_pair);
+
 	return my_list;
 }
