@@ -2,14 +2,15 @@
  *  smap.c - Report overall state the system
  *****************************************************************************
  *  Copyright (C) 2004-2007 The Regents of the University of California.
- *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2011 Lawrence Livermore National Security.
+ *  Copyright (C) 2011 SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  Written by Danny Auble <da@llnl.gov>
+ *  Written by Danny Auble <da@schedmd.com>
  *
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -52,12 +53,16 @@ static int min_screen_width = 72;
 int text_line_cnt = 0;
 
 smap_parameters_t params;
+smap_system_t *smap_system_ptr;
 
 int quiet_flag = 0;
 int grid_line_cnt = 0;
 int max_display;
 int resize_screen = 0;
 
+int *dim_size = NULL;
+char letters[62];
+char colors[6];
 int main_xcord = 1;
 int main_ycord = 1;
 WINDOW *grid_win = NULL;
@@ -67,9 +72,11 @@ WINDOW *text_win = NULL;
  * Functions *
  ************/
 
-static int _get_option();
+static int  _get_option(void);
+static void _init_colors(void);
 static void *_resize_handler(int sig);
-static int _set_pairs();
+static int  _set_pairs(void);
+static void _smap_exit(int rc);
 
 int main(int argc, char *argv[])
 {
@@ -84,8 +91,6 @@ int main(int argc, char *argv[])
 	int end = 0;
 	int i;
 	int rc;
-	int mapset = 0;
-	//char *name;
 
 	log_init(xbasename(argv[0]), opts, SYSLOG_FACILITY_DAEMON, NULL);
 	parse_command_line(argc, argv);
@@ -94,97 +99,69 @@ int main(int argc, char *argv[])
 		log_alter(opts, SYSLOG_FACILITY_USER, NULL);
 	}
 
-	if(params.cluster_dims == 4) {
-		/* FIX ME: smap doesn't do anything correctly with
-		   more than 3 dims yet.
-		*/
-	} else if(params.cluster_dims == 3)
+	if (params.cluster_dims == 4) {
+		min_screen_width = 92;
+	} else if (params.cluster_dims == 3)
 		min_screen_width = 92;
 
-	while (slurm_load_node((time_t) NULL, &new_node_ptr, SHOW_ALL)) {
-		error_code = slurm_get_errno();
-		printf("slurm_load_node: %s\n", slurm_strerror(error_code));
-		if (params.display == COMMANDS) {
-			new_node_ptr = NULL;
-			break;		/* just continue */
-		}
-		if (params.iterate == 0)
-			exit(1);
-		sleep(10);	/* keep trying to reconnect */
-	}
-
-	ba_init(new_node_ptr, 0);
-
-	if(params.resolve) {
-
-#ifdef HAVE_BG_FILES
-		if (!have_db2) {
-			printf("Required libraries can not be found "
-			       "to access the Bluegene system.\nPlease "
-			       "set your LD_LIBRARY_PATH correctly to "
-			       "point to them.\n");
-			goto part_fini;
-		}
-
-		if(!mapset)
-			mapset = set_bp_map();
-		if(params.resolve[0] != 'R') {
-			i = strlen(params.resolve);
-			i -= 3;
-			if(i<0) {
-				printf("No real block was entered\n");
-				goto part_fini;
+	/* no need for this if you are resolving */
+	if (!params.resolve) {
+		while (slurm_load_node((time_t) NULL,
+				       &new_node_ptr, SHOW_ALL)) {
+			error_code = slurm_get_errno();
+			printf("slurm_load_node: %s\n",
+			       slurm_strerror(error_code));
+			if (params.display == COMMANDS) {
+				new_node_ptr = NULL;
+				break;		/* just continue */
 			}
-			char *rack_mid = find_bp_rack_mid(params.resolve+i);
-			if(rack_mid)
-				printf("X=%c Y=%c Z=%c resolves to %s\n",
-				       params.resolve[X+i],
-				       params.resolve[Y+i],
-				       params.resolve[Z+i],
-				       rack_mid);
-			else
-				printf("X=%c Y=%c Z=%c has no resolve\n",
-				       params.resolve[X+i],
-				       params.resolve[Y+i],
-				       params.resolve[Z+i]);
-
-		} else {
-			uint16_t *coord = find_bp_loc(params.resolve);
-			if(coord)
-				printf("%s resolves to X=%d Y=%d Z=%d\n",
-				       params.resolve,
-				       coord[X], coord[Y], coord[Z]);
-			else
-				printf("%s has no resolve.\n",
-				       params.resolve);
+			if (params.iterate == 0)
+				exit(1);
+			sleep(10);	/* keep trying to reconnect */
 		}
-part_fini:
-#else
-		printf("Must be physically on a BG System to resolve.\n");
-#endif
-		ba_fini();
-		xfree(params.resolve);
-		exit(0);
 	}
-	if(!params.commandline) {
+
+#ifdef HAVE_BG
+	bg_configure_ba_init(new_node_ptr, 0);
+#endif
+	if (dim_size == NULL) {
+		dim_size = get_cluster_dims(new_node_ptr);
+		if ((dim_size == NULL) || (dim_size[0] < 1))
+			fatal("Invalid system dimensions");
+	}
+	_init_colors();
+
+	if (params.resolve) {
+		char *ret_str = resolve_mp(params.resolve);
+		if (ret_str) {
+			printf("%s", ret_str);
+			xfree(ret_str);
+		}
+		_smap_exit(0);	/* Calls exit(), no return */
+	}
+	if (!params.commandline) {
 		int check_width = min_screen_width;
+
+		init_grid(new_node_ptr);
+
 		signal(SIGWINCH, (void (*)(int))_resize_handler);
 		initscr();
 
-		if(params.cluster_dims == 4) {
-			/* FIX ME: smap doesn't do anything correctly with
-			   more than 3 dims yet.
-			*/
-		} else if(params.cluster_dims == 3) {
-			height = DIM_SIZE[Y] * DIM_SIZE[Z] + DIM_SIZE[Y] + 3;
-			width = DIM_SIZE[X] + DIM_SIZE[Z] + 3;
+		if (params.cluster_dims == 4) {
+			height = dim_size[2] * dim_size[3] + dim_size[2] + 3;
+			width = (dim_size[1] + dim_size[3] + 1) * dim_size[0]
+				 + 2;
+			check_width += width;
+		} else if (params.cluster_dims == 3) {
+			height = dim_size[1] * dim_size[2] + dim_size[1] + 3;
+			width = dim_size[0] + dim_size[2] + 3;
 			check_width += width;
 		} else {
 			height = 10;
 			width = COLS;
 		}
 
-	        if (COLS < check_width || LINES < height) {
+	        if ((COLS < check_width) || (LINES < height)) {
 			endwin();
 			error("Screen is too small make sure the screen "
 			      "is at least %dx%d\n"
@@ -193,28 +170,27 @@ part_fini:
 			      height,
 			      COLS,
 			      LINES);
-			ba_fini();
-			exit(1);
+			_smap_exit(1);	/* Calls exit(), no return */
 		}
 
 		raw();
 		keypad(stdscr, TRUE);
 		noecho();
 		cbreak();
-		curs_set(1);
+		curs_set(0);
 		nodelay(stdscr, TRUE);
 		start_color();
 		_set_pairs();
 
 		grid_win = newwin(height, width, starty, startx);
 		max_display = grid_win->_maxy * grid_win->_maxx;
-		//scrollok(grid_win, TRUE);
 
-		if(params.cluster_dims == 4) {
-			/* FIX ME: smap doesn't do anything correctly with
-			   more than 3 dims yet.
-			*/
-		} else if(params.cluster_dims == 3) {
+		if (params.cluster_dims == 4) {
+			startx = width;
+			COLS -= 2;
+			width = COLS - width;
+			height = LINES;
+		} else if (params.cluster_dims == 3) {
 			startx = width;
 			COLS -= 2;
 			width = COLS - width;
@@ -228,22 +204,21 @@ part_fini:
 		text_win = newwin(height, width, starty, startx);
         }
 	while (!end) {
-		if(!params.commandline) {
+		if (!params.commandline) {
 			_get_option();
-		redraw:
-
+redraw:
 			clear_window(text_win);
 			clear_window(grid_win);
-			move(0,0);
+			move(0, 0);
 
-			init_grid(new_node_ptr);
 			main_xcord = 1;
 			main_ycord = 1;
 		}
 
-		if(!params.no_header)
+		if (!params.no_header)
 			print_date();
 
+		clear_grid();
 		switch (params.display) {
 		case JOBS:
 			get_job();
@@ -255,43 +230,35 @@ part_fini:
 			get_slurm_part();
 			break;
 		case COMMANDS:
-			if(params.cluster_flags & CLUSTER_FLAG_BG) {
-				if(!mapset) {
-					mapset = set_bp_map();
-					wclear(text_win);
-					//doupdate();
-					//move(0,0);
-				}
-				get_command();
-			} else {
-				error("Must be on a BG SYSTEM to "
-				      "run this command");
-				if(!params.commandline)
-					endwin();
-				ba_fini();
-				exit(1);
-			}
+#ifdef HAVE_BG
+			wclear(text_win);
+			get_command();
+#else
+			error("Must be on a real BG SYSTEM to "
+			      "run this command");
+			if (!params.commandline)
+				endwin();
+			_smap_exit(1);	/* Calls exit(), no return */
+#endif
 			break;
 		case BGPART:
-			if(params.cluster_flags & CLUSTER_FLAG_BG)
+			if (params.cluster_flags & CLUSTER_FLAG_BG)
 				get_bg_part();
 			else {
 				error("Must be on a BG SYSTEM to "
 				      "run this command");
-				if(!params.commandline)
+				if (!params.commandline)
 					endwin();
-				ba_fini();
-				exit(1);
+				_smap_exit(1);	/* Calls exit(), no return */
 			}
 			break;
 		}
 
-		if(!params.commandline) {
-			//wscrl(grid_win,-1);
+		if (!params.commandline) {
 			box(text_win, 0, 0);
 			wnoutrefresh(text_win);
 
-			print_grid(grid_line_cnt * (grid_win->_maxx-1));
+			print_grid();
 			box(grid_win, 0, 0);
 			wnoutrefresh(grid_win);
 
@@ -316,7 +283,7 @@ part_fini:
 					&new_node_ptr, SHOW_ALL);
 			}
 			if (error_code && (quiet_flag != 1)) {
-				if(!params.commandline) {
+				if (!params.commandline) {
 					mvwprintw(
 						text_win,
 						main_ycord,
@@ -335,9 +302,8 @@ part_fini:
 
 		if (params.iterate) {
 			for (i = 0; i < params.iterate; i++) {
-
 				sleep(1);
-				if(!params.commandline) {
+				if (!params.commandline) {
 					if ((rc = _get_option()) == 1)
 						goto redraw;
 					else if (resize_screen) {
@@ -351,17 +317,58 @@ part_fini:
 
 	}
 
-	if(!params.commandline) {
+	if (!params.commandline) {
 		nodelay(stdscr, FALSE);
 		getch();
 		endwin();
 	}
-	ba_fini();
 
-	exit(0);
+	_smap_exit(0);	/* Calls exit(), no return */
+	exit(0);	/* Redundant, but eliminates compiler warning */
 }
 
-static int _get_option()
+static void _init_colors(void)
+{
+	int x, y, z;
+	/* make the letters array only contain letters upper and lower (62) */
+	y = 'A';
+	for (x = 0; x < 62; x++) {
+		if (y == '[')
+			y = 'a';
+		else if (y == '{')
+			y = '0';
+		else if (y == ':')
+			y = 'A';
+		letters[x] = y;
+		y++;
+	}
+
+	z = 1;
+	for (x = 0; x < 6; x++) {
+		if (z == 4)
+			z++;
+		colors[x] = z;
+		z++;
+	}
+}
+
+/* Variation of exit() that releases memory as needed for memory leak test */
+static void _smap_exit(int rc)
+{
+#ifdef MEMORY_LEAK_DEBUG
+	free_grid();
+
+#ifdef HAVE_BG
+	bg_configure_ba_fini();
+#endif
+
+#endif
+	if (!params.commandline)
+		curs_set(1);
+	exit(rc);
+}
+
+static int _get_option(void)
 {
 	int ch;
 
@@ -371,28 +378,26 @@ static int _get_option()
 	case '-':
 	case '_':
 		text_line_cnt++;
-	return 1;
-	break;
+		return 1;
+		break;
 	case KEY_LEFT:
 	case '=':
 	case '+':
 		text_line_cnt--;
-	if(text_line_cnt<0) {
-		text_line_cnt = 0;
-		return 0;
-
-	}
-	return 1;
-	break;
-
+		if (text_line_cnt < 0) {
+			text_line_cnt = 0;
+			return 0;
+		}
+		return 1;
+		break;
 	case 'H':
 	case 'h':
-		if(params.all_flag)
+		if (params.all_flag)
 			params.all_flag = 0;
 		else
 			params.all_flag = 1;
 	return 1;
-	break;
+		break;
 	case 's':
 		text_line_cnt = 0;
 		grid_line_cnt = 0;
@@ -412,7 +417,7 @@ static int _get_option()
 		return 1;
 		break;
 	case 'b':
-		if(params.cluster_flags & CLUSTER_FLAG_BG) {
+		if (params.cluster_flags & CLUSTER_FLAG_BG) {
 			text_line_cnt = 0;
 			grid_line_cnt = 0;
 			params.display = BGPART;
@@ -420,16 +425,16 @@ static int _get_option()
 		}
 		break;
 	case 'c':
-		if(params.cluster_flags & CLUSTER_FLAG_BG) {
+		if (params.cluster_flags & CLUSTER_FLAG_BG) {
 			params.display = COMMANDS;
 			return 1;
 		}
 		break;
 	case 'u':
 	case KEY_UP:
-		if(!(params.cluster_flags & CLUSTER_FLAG_BG)) {
+		if (!(params.cluster_flags & CLUSTER_FLAG_BG)) {
 			grid_line_cnt--;
-			if(grid_line_cnt<0) {
+			if (grid_line_cnt<0) {
 				grid_line_cnt = 0;
 				return 0;
 			}
@@ -438,29 +443,28 @@ static int _get_option()
 	break;
 	case 'd':
 	case KEY_DOWN:
-		if(!(params.cluster_flags & CLUSTER_FLAG_BG)) {
+		if (!(params.cluster_flags & CLUSTER_FLAG_BG)) {
 			grid_line_cnt++;
-			if((((grid_line_cnt-2) * (grid_win->_maxx-1)) +
-			    max_display) > DIM_SIZE[X]) {
+			if ((((grid_line_cnt-2) * (grid_win->_maxx-1)) +
+			    max_display) > dim_size[0]) {
 				grid_line_cnt--;
 				return 0;
 			}
 			return 1;
 		}
-	break;
+		break;
 	case 'q':
 	case '\n':
 		endwin();
-	ba_fini();
-	exit(0);
-	break;
+		_smap_exit(0);	/* Calls exit(), no return */
+		break;
 	}
 	return 0;
 }
 
 static void *_resize_handler(int sig)
 {
-	int startx=0, starty=0;
+	int startx = 0, starty = 0;
 	int height = 40, width = 100;
 	int check_width = min_screen_width;
 	main_ycord = 1;
@@ -473,19 +477,19 @@ static void *_resize_handler(int sig)
 	delwin(text_win);
 
 	endwin();
-	COLS=0;
-	LINES=0;
+	COLS = 0;
+	LINES = 0;
 	initscr();
 	doupdate();	/* update now to make sure we get the new size */
-	getmaxyx(stdscr,LINES,COLS);
+	getmaxyx(stdscr, LINES, COLS);
 
-	if(params.cluster_dims == 4) {
-		/* FIX ME: smap doesn't do anything correctly with
-		   more than 3 dims yet.
-		*/
-	} else if(params.cluster_dims == 3) {
-		height = DIM_SIZE[Y] * DIM_SIZE[Z] + DIM_SIZE[Y] + 3;
-		width = DIM_SIZE[X] + DIM_SIZE[Z] + 3;
+	if (params.cluster_dims == 4) {
+		height = dim_size[2] * dim_size[3] + dim_size[2] + 3;
+		width = (dim_size[1] + dim_size[3] + 1) * dim_size[0];
+		check_width += width;
+	} else if (params.cluster_dims == 3) {
+		height = dim_size[1] * dim_size[2] + dim_size[1] + 3;
+		width = dim_size[0] + dim_size[2] + 3;
 		check_width += width;
 	} else {
 		height = 10;
@@ -497,18 +501,18 @@ static void *_resize_handler(int sig)
 		error("Screen is too small make sure "
 		      "the screen is at least %dx%d\n"
 		      "Right now it is %dx%d\n", width, height, COLS, LINES);
-		ba_fini();
-		exit(0);
+		_smap_exit(0);	/* Calls exit(), no return */
 	}
 
 	grid_win = newwin(height, width, starty, startx);
 	max_display = grid_win->_maxy * grid_win->_maxx;
 
-	if(params.cluster_dims == 4) {
-		/* FIX ME: smap doesn't do anything correctly with
-		   more than 3 dims yet.
-		*/
-	} else if(params.cluster_dims == 3) {
+	if (params.cluster_dims == 4) {
+		startx = width;
+		COLS -= 2;
+		width = COLS - width;
+		height = LINES;
+	} else if (params.cluster_dims == 3) {
 		startx = width;
 		COLS -= 2;
 		width = COLS - width;
@@ -533,16 +537,17 @@ static void *_resize_handler(int sig)
 		get_slurm_part();
 		break;
 	case COMMANDS:
-		if(params.cluster_flags & CLUSTER_FLAG_BG)
-			get_command();
+#ifdef HAVE_BG
+		get_command();
+#endif
 		break;
 	case BGPART:
-		if(params.cluster_flags & CLUSTER_FLAG_BG)
+		if (params.cluster_flags & CLUSTER_FLAG_BG)
 			get_bg_part();
 		break;
 	}
 
-	print_grid(grid_line_cnt * (grid_win->_maxx-1));
+	print_grid();
 	box(text_win, 0, 0);
 	box(grid_win, 0, 0);
 	wnoutrefresh(text_win);
@@ -552,7 +557,7 @@ static void *_resize_handler(int sig)
 	return NULL;
 }
 
-static int _set_pairs()
+static int _set_pairs(void)
 {
 	int x;
 
